/*
 * raw.ino
 * Reads raw timing pulses and sends them to the EUR7721KD0.exe program. 
 * This program sketch runs on a Arduino board.  It is controlled by the EUR7721KD0.exe program running on a Windows 7 to 10 PC.
 * The EUR7721KD0.exe program and Arduino sketches can be downloaded from http://www.stevematt.f9.co.uk/EUR7721KD0.
 * To use the IR Monitor mode in the EUR7721KD0.exe program you must have this sketch running on your Arduino.
 * This allows the EUR7721KD0.exe program to display the codes from your remote as bars showing the marks and spaces.
 * If you want to see the pulse timings for your remote open the Serial Monitor of your Arduino CC program with the port speed set to 115200.
 * Point your remote at your Arduino board, presss a button on your remote and you will see the pulse timing figures.
 * This sketch has only been tested on a Arduino UNO board.
 * 
 * This sketch uses Ken Shirriff's IRRemote library.
 * To compile this sketch you must install that library.
 * For more info see http://www.righto.com/2009/08/multi-protocol-infrared-remote-library.html
 * The most recent code is at github.com/shirriff/Arduino-IRremote. 
 * An IR transmitter LED must be connected to your Arduino PWM pin 3.  You must use a 100 ohm resistor in series with the LED.
 *
 * The EUR7721KD0.exe Windows program sends key codes to a Arduino via a USB cable when you click on the buttons of the remote image.
 * The MyRemote.ino sketch running in a Arduino uses the key codes to look up a infra-red remote code and sends that out via a infra-red led.
 * If you have an old spare remote you can break it open and use the infra-red LED from that or buy a new LED from a some where.
 * The com port speed settings in the the EUR7721KD0.exe Windows program must be the same as in the Serial.begin(115200); 
 * statement in this sketch.  If you want you can use other speeds but they must both match.
 * 
 * You can control other kit by changing the infra-red codes in "long TheCode[54]{".
 * Have a look at Ken Shirriff's IRRemote library for how to read remote infra-red codes.
 */
#include <IRremote.h>
 
#define PanasonicAddress      0x4004     // Panasonic address (Pre data) 
#define PanasonicPower        0x100BCBD  // Panasonic Power button

void  MydumpRaw (decode_results *results);
int recvPin = 11;

//------------------------------------------------------------------------------
// Include the IRremote library header
//
#include <IRremote.h>

//------------------------------------------------------------------------------
// Tell IRremote which Arduino pin is connected to the IR Receiver (TSOP4838)
//

IRrecv irrecv(recvPin);

//+=============================================================================
// Configure the Arduino
//
void  setup ( )
{
  Serial.begin(115200);   // Status message will be sent to PC at 9600 baud
  //irrecv.enableIRIn();  // Start the receiver

    irparams.recvpin = recvPin;
  irparams.blinkflag = 0;
  cli();
  // setup pulse clock timer interrupt
  //Prescale /8 (16M/8 = 0.5 microseconds per tick)
  // Therefore, the timer interval can range from 0.5 to 128 microseconds
  // depending on the reset value (255 to 0)
  TIMER_CONFIG_NORMAL();

  //Timer2 Overflow Interrupt Enable
  TIMER_ENABLE_INTR;

  TIMER_RESET;

  sei();  // enable interrupts

  // initialize state machine variables
  irparams.rcvstate = STATE_IDLE;
  irparams.rawlen = 0;

  // set pin modes
  pinMode(irparams.recvpin, INPUT);
}


//+=============================================================================
// Dump out the decode_results structure.
//
void  MydumpRaw (decode_results *results)
{

  //volatile unsigned int *rawbuf; // Raw intervals in .5 us ticks
  
  // Print Raw data
if( irparams.rawlen <1)
{
  return;
}
  
 // Serial.print("Timing[");
  //Serial.print(irparams.rawlen, DEC);
  //Serial.println("]: ");

  //for (int i = 1;  i < results->rawlen;  i++) {
 for (int i = 1;  i <irparams.rawlen;  i++) {
    unsigned long  x = irparams.rawbuf[i] * USECPERTICK;
       Serial.print(x, DEC);
      Serial.print(" ");
     }
 
  Serial.print("x");                    // Newline
  //irparams.rawlen = 0;

  
}


//+=============================================================================
// The repeating section of the code
//
void  loop ( )
{
  int thelen=0;
  decode_results  results;        // Somewhere to store the results

  if ( irparams.rcvstate == STATE_STOP)
  {
    TIMER_DISABLE_INTR;
 // Grab an IR code
    //dumpInfo(&results);           // Output the results
    thelen = results.rawlen;
    if( thelen > 0)
    {
      MydumpRaw(&results);            // Output the results in RAW format
    }
    irrecv.decode(&results); 
    Serial.print("Number of highs and lows ");
    Serial.print(results.rawlen, DEC);
    Serial.print("  ");
    Serial.print("Code      : ");
    Serial.print(results.value, HEX);
    Serial.print(" Y");               //Y flags to the EUR7721KD0.exe program the end of the Code text.
    //dumpInfo(&results);           // Output the results
    //Serial.println("");           // Blank line between entries
    //irrecv.resume();              // Prepare for the next value
    
      TIMER_ENABLE_INTR;
  // initialize state machine variables
  irparams.rcvstate = STATE_IDLE;
  irparams.rawlen = 0;
  }
}
//+=============================================================================
// Dump out the decode_results structure.
//
void  dumpInfo (decode_results *results)
{
  // Check if the buffer overflowed
  if (results->overflow) {
    Serial.println("IR code too long. Edit IRremoteInt.h and increase RAWLEN");
   // return;
  }

  // Show Encoding standard
  Serial.print("Encoding  : ");
  encoding(results);
  Serial.println("");

  // Show Code & length
  Serial.print("Code      : ");
  ircode(results);
  Serial.print(" (");
  Serial.print(results->bits, DEC);
  Serial.println(" bits)");

  Serial.println(" Y");       //Flag for the EUR7721KD0.exe program to indicate the end of the dumpinfo texts.
}
//+=============================================================================
// Display IR code
//
void  ircode (decode_results *results)
{
  // Panasonic has an Address
  if (results->decode_type == PANASONIC) {
    Serial.print(results->address, HEX);
    Serial.print(":");
  }

  // Print Code
  Serial.print(results->value, HEX);
}


//+=============================================================================
// Display encoding type
//
void  encoding (decode_results *results)
{
  switch (results->decode_type) {
    default:
    case UNKNOWN:      Serial.print("UNKNOWN");       break ;
    case NEC:          Serial.print("NEC");           break ;
    case SONY:         Serial.print("SONY");          break ;
    case RC5:          Serial.print("RC5");           break ;
    case RC6:          Serial.print("RC6");           break ;
    case DISH:         Serial.print("DISH");          break ;
    case SHARP:        Serial.print("SHARP");         break ;
    case JVC:          Serial.print("JVC");           break ;
    case SANYO:        Serial.print("SANYO");         break ;
    case MITSUBISHI:   Serial.print("MITSUBISHI");    break ;
    case SAMSUNG:      Serial.print("SAMSUNG");       break ;
    case LG:           Serial.print("LG");            break ;
    case WHYNTER:      Serial.print("WHYNTER");       break ;
    case AIWA_RC_T501: Serial.print("AIWA_RC_T501");  break ;
    case PANASONIC:    Serial.print("PANASONIC");     break ;
    case DENON:        Serial.print("Denon");         break ;
  }
}


